/*
 Name:    Arduino MEGA communication.ino
 Created: 19.03.2019
 Author:  Even L, Jarle N, Thomas K

 Edited:     13.04.2021
 Edited by:  Mathias D, Anne V, Maren A
 
 This program recieves data from the OpenMV H7 CAMERAS (1-5) via an UART communication protocoll and transmits it further to the
 Arduino UNO Communication via the I2C-bus.
*/

#include <Wire.h> //Includes the required library
#include <SoftwareSerial.h>

#define rx4 10
#define tx4 9

SoftwareSerial Serial4(rx4, tx4);  //Digital input RX TX CAMERA 5

String trainData[] = {"","","","",""}; //Data


void setup(){
  Wire.begin(4);                //Joins i2c bus with address #2
  Serial.begin(19200);          //Starts the serial ports with baudrate 19200
  Serial1.begin(19200);
  Serial2.begin(19200);
  Serial3.begin(19200);
  Serial4.begin(19200);
}

void loop(){
  //Read input [0]
  if(Serial.available() > 0){ 
    char byteRead = Serial.read(); 
    ReadTrainData(0,byteRead);
  }
  
  //Read input [1]
  if(Serial1.available() > 0){ 
    char byteRead = Serial1.read(); 
    ReadTrainData(1,byteRead);
  }
  
  //Read input [2]
  if(Serial2.available() > 0){ 
    char byteRead = Serial2.read(); 
    ReadTrainData(2,byteRead);
  }
  
  //Read input [3]
  if(Serial3.available() > 0){ 
    char byteRead = Serial3.read(); 
    ReadTrainData(3,byteRead);
  }

    //Read input [4]
  if(Serial4.available() > 0){
    char byteRead = Serial4.read();
    ReadTrainData(4,byteRead);
  }
}

//Read the incoming data and update the trainData array
//Send the data if it ends with "\n" and reset the string
void ReadTrainData(int i, char byteRead){
  trainData[i] += byteRead;

  if(trainData[i].endsWith("\n")){
    WriteToMaster(trainData[i]);
    trainData[i] = "";  
  }
}

//Print finished trainData to Master via I2C protocol
void WriteToMaster(String data){
    Wire.beginTransmission(2);
    Wire.print(data);
    Wire.endTransmission();
}
