# Created:  16.04.2021
# Author: Mathias D, Maren A, Anne V
# This program reads tags on the trains and detects obstacles
#

import time
from pyb import UART
import sensor, image, time, math, pyb, ustruct, os

# Camera setup
sensor.reset()
sensor.set_hmirror(1)
sensor.set_vflip(1)
sensor.set_pixformat(sensor.GRAYSCALE)
sensor.set_framesize(sensor.VGA)
sensor.set_windowing((200,220))
sensor.set_auto_whitebal(False)


# Settings
camID = 1
maxTrainTag = 10    #prevents ghost tags


# Border corners (x,y)
TL = [80,70]
BL = [-20,220]

TR = [140,70]
BR = [250,220]

TC = [int(TL[0] + (TR[0] - TL[0])/2), int(TL[1] + (TR[1]-TL[1])/2)]
BC = [int(BL[0] + (BR[0] - BL[0])/2), int(BL[1] + (BR[1]-BL[1])/2)]

imgRegion = [0,0,sensor.get_windowing()[2], sensor.get_windowing()[3]]


# Blob Data
lightThresholds = (50, 255)
areaThreshold = 200
pixelsThreshold  = 50

left = False
right = False
isTrain = False

frameCount = 0
framesToSave = 300      # number of frames to save new background image (i frame = 1 ms)


# Communication
uart = UART(3, 19200)


def SaveBackgroundImage():
    if not "temp" in os.listdir():
        os.mkdir("temp")

    sensor.snapshot().save("temp/bg.bmp")
    print("Saved background image")


def Overlap(x,y,w,h):
    return IsLeftSide(x, y + h, BR,TR) and not IsLeftSide(x + w, y + h,BL,TL) and IsLeftSide(x, y, BL,BR) and not IsLeftSide(x, y + h, TL,TR)


def IsLeftSide(x,y,a,b):
    d = (b[0] - a[0]) * (y - a[1]) - (b[1] - a[1]) * (x - a[0])
    return d < 0

def SendData(trainID, x, y):
    text = str(camID) + ";" + str(trainID) + ";" + str(-x) + ";" + str(y)+ "|" + str(int(left)) + ";" + str(int(right))+"\n"
    print(text)
    uart.write(text)

def DrawBorder():
    lineColor = 0

    if left or right:
        lineColor = 255

    main.draw_line(BL[0], BL[1], TL[0], TL[1], color = lineColor)
    main.draw_line(BR[0], BR[1], TR[0], TR[1], color = lineColor)
    main.draw_line(TL[0], TL[1], TR[0], TR[1], color = lineColor)
    main.draw_line(BL[0], BL[1], BR[0], BR[1], color = lineColor)
    main.draw_line(BC[0], BC[1], TC[0], TC[1], color = lineColor)
    main.draw_cross(TL)
    main.draw_cross(BL)
    main.draw_cross(TR)
    main.draw_cross(BR)


 # Make a temp directory
print("About to save background image...")
sensor.skip_frames(time = 2000) # Give the user time to get ready.
SaveBackgroundImage()

print("Camera is ready.")


while(True):
    time.sleep_ms(100)

    main = sensor.snapshot()
    img = main.copy(imgRegion)

    frameCount += 1

    img.difference("temp/bg.bmp")
    img.gamma_corr(gamma=0.8, contrast=3.0, brightness=0)

    img.draw_line(0,0,200,0,0, thickness = 160)
    img.draw_line(-30,150,70,0,0, thickness = 80)

    left = False
    right = False

    for blob in img.find_blobs([lightThresholds], pixels_threshold=pixelsThreshold, area_threshold=areaThreshold, merge=True):
        if Overlap(blob.x(), blob.y(), blob.w(),blob.h()):
            #main.draw_rectangle(blob.rect()) #debugging only

            if not left:
                left = IsLeftSide(blob.x(), blob.y(), BC, TC) or IsLeftSide(blob.x(), blob.y() + blob.h(), BC, TC)

            if not right:
                right = not IsLeftSide(blob.x() + blob.w(), blob.y() + blob.h(), BC, TC) or not IsLeftSide(blob.x() + blob.w(), blob.y(), BC, TC)

    isTrain = False

    for tag in main.find_apriltags(families = image.TAG16H5):
        if tag.id() < maxTrainTag:
            if not isTrain:
                isTrain = True

            SendData(tag.id(), tag.x_translation(), tag.z_translation())

    if isTrain or (not left and not right):
        frameCount = 0

    if not isTrain and (left or right):
        SendData(-1, 0, 0)

    if frameCount > framesToSave and not isTrain:
        frameCount = 0
        SaveBackgroundImage()

    DrawBorder()
